@@ -29,7 +29,8 @@ from pymobiledevice3.cli.syslog import cli as syslog_cli
 from pymobiledevice3.cli.webinspector import cli as webinspector_cli
 from pymobiledevice3.exceptions import NoDeviceConnectedError, NotPairedError, UserDeniedPairingError, \
     PairingDialogResponsePendingError, SetProhibitedError, MissingValueError, DeviceHasPasscodeSetError, \
-    DeveloperModeError, UsbmuxConnectionError
+    DeveloperModeError, UsbmuxConnectionError, NoDeviceSelectedError, MessageNotSupportedError, InternalError, \
+    DeveloperModeIsNotEnabledError, InvalidServiceError
 
 coloredlogs.install(level=logging.INFO)
 
@@ -73,6 +74,18 @@ def cli():
         logger.error(f'Failed to enable developer-mode. Error: {e}')
     except UsbmuxConnectionError:
         logger.error('Failed to connect to usbmuxd socket. Make sure it\'s running.')
+    except MessageNotSupportedError:
+        logger.error('Message not supported for this iOS version')
+    except InternalError:
+        logger.error('Internal Error')
+    except DeveloperModeIsNotEnabledError:
+        logger.error('Developer Mode is disabled. You can try to enable it using: '
+                     'python3 -m pymobiledevice3 amfi enable-developer-mode')
+    except InvalidServiceError:
+        logger.error('Failed to access an invalid lockdown service, possibly from DeveloperDiskImage.dmg or a Cryptex. '
+                     'You may try: python3 -m pymobiledevice3 mounter auto-mount')
+    except NoDeviceSelectedError:
+        return
     except BrokenPipeError:
         traceback.print_exc()
 
