@@ -6,9 +6,13 @@ import uuid
 
 import click
 import coloredlogs
+import inquirer
+from inquirer.themes import GreenPassion
 from pygments import highlight, lexers, formatters
 
+from pymobiledevice3.exceptions import NoDeviceSelectedError
 from pymobiledevice3.lockdown import LockdownClient
+from pymobiledevice3.usbmux import select_devices_by_connection_type
 
 
 def default_json_encoder(obj):
@@ -42,6 +46,20 @@ def wait_return():
 UDID_ENV_VAR = 'PYMOBILEDEVICE3_UDID'
 
 
+class DeviceInfo:
+    def __init__(self, lockdown_client: LockdownClient):
+        self.lockdown_client = lockdown_client
+        self.product_version = self.lockdown_client.product_version
+        self.serial = self.lockdown_client.identifier
+        self.display_name = self.lockdown_client.display_name
+
+    def __str__(self):
+        if self.display_name is None:
+            return f'Unknown device, ios version: {self.product_version}, serial: {self.serial}'
+        else:
+            return f'{self.display_name}, ios version: {self.product_version}, serial: {self.serial}'
+
+
 class Command(click.Command):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -57,7 +75,26 @@ class Command(click.Command):
         if '_PYMOBILEDEVICE3_COMPLETE' in os.environ:
             # prevent lockdown connection establishment when in autocomplete mode
             return
-        return LockdownClient(serial=value)
+
+        if value is not None:
+            return LockdownClient(serial=value)
+
+        devices = select_devices_by_connection_type(connection_type='USB')
+        if len(devices) <= 1:
+            return LockdownClient()
+
+        devices_options = []
+        for device in devices:
+            lockdown_client = LockdownClient(serial=device.serial)
+            device_info = DeviceInfo(lockdown_client)
+            devices_options.append(device_info)
+
+        device_question = [inquirer.List('device', message='choose device', choices=devices_options, carousel=True)]
+        try:
+            result = inquirer.prompt(device_question, theme=GreenPassion(), raise_keyboard_interrupt=True)
+            return result['device'].lockdown_client
+        except KeyboardInterrupt as e:
+            raise NoDeviceSelectedError from e
 
 
 class CommandWithoutAutopair(Command):
