@@ -186,7 +186,7 @@ def show_dirlist(device_info: DeviceInfo, dirname, recursive=False):
 
 
 @dvt.command('ls', cls=Command)
-@click.argument('path', type=click.Path(exists=False))
+@click.argument('path', type=click.Path(exists=False, readable=False))
 @click.option('-r', '--recursive', is_flag=True)
 def ls(lockdown: LockdownClient, path, recursive):
     """ List directory. """
@@ -897,3 +897,13 @@ def check_out(lockdown: LockdownClient):
     """ owner check-out """
     with DtDeviceArbitration(lockdown) as device_arbitration:
         device_arbitration.check_out()
+
+
+@dvt.command('har', cls=Command)
+def dvt_har(lockdown: LockdownClient):
+    """ enable har-logging """
+    with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
+        print('> Press Ctrl-C to abort')
+        with ActivityTraceTap(dvt, enable_http_archive_logging=True) as tap:
+            while True:
+                tap.channel.receive_message()
