@@ -1,9 +1,12 @@
 import asyncio
 import logging
 import plistlib
+import socket
 import ssl
 import struct
 import time
+from enum import Enum, auto
+from typing import Optional, Mapping
 
 import IPython
 from pygments import highlight, lexers, formatters
@@ -11,7 +14,7 @@ from pygments import highlight, lexers, formatters
 from pymobiledevice3 import usbmux
 from pymobiledevice3.exceptions import ConnectionFailedError, PyMobileDevice3Exception, ConnectionTerminatedError, \
     NoDeviceConnectedError
-from pymobiledevice3.usbmux import select_device
+from pymobiledevice3.usbmux import select_device, MuxDevice
 
 SHELL_USAGE = """
 # This shell allows you to communicate directly with every service layer behind the lockdownd daemon.
@@ -55,34 +58,58 @@ def create_context(certfile, keyfile=None):
     return context
 
 
+class Medium(Enum):
+    TCP = auto()
+    USBMUX = auto()
+
+
 class ServiceConnection(object):
     """ wrapper for usbmux tcp-relay connections """
 
-    def __init__(self, socket):
+    def __init__(self, sock: socket.socket, mux_device: MuxDevice = None):
         self.logger = logging.getLogger(__name__)
-        self.socket = socket
-        self._reader = None  # type: asyncio.StreamReader
-        self._writer = None  # type: asyncio.StreamWriter
+        self.socket = sock
+
+        # usbmux connections contain additional information associated with the current connection
+        self.mux_device = mux_device
+
+        self._reader = None  # type: Optional[asyncio.StreamReader]
+        self._writer = None  # type: Optional[asyncio.StreamWriter]
 
     @staticmethod
-    def create(udid: str, port: int, connection_type=None):
+    def create_using_tcp(hostname: str, port: int) -> 'ServiceConnection':
+        sock = socket.socket()
+        sock.connect((hostname, port))
+        return ServiceConnection(sock)
+
+    @staticmethod
+    def create_using_usbmux(udid: Optional[str], port: int, connection_type: str = None) -> 'ServiceConnection':
         target_device = select_device(udid, connection_type=connection_type)
         if target_device is None:
+            if udid:
+                raise ConnectionFailedError()
             raise NoDeviceConnectedError()
         try:
-            socket = target_device.connect(port)
+            sock = target_device.connect(port)
         except usbmux.MuxException:
             raise ConnectionFailedError(f'Connection to device port {port} failed')
 
-        return ServiceConnection(socket)
+        return ServiceConnection(sock, mux_device=target_device)
 
-    def setblocking(self, blocking: bool):
+    @staticmethod
+    def create(medium: Medium, identifier: str, port: int, connection_type: str = None) -> 'ServiceConnection':
+        if medium == Medium.TCP:
+            return ServiceConnection.create_using_tcp(identifier, port)
+        else:
+            return ServiceConnection.create_using_usbmux(identifier, port, connection_type=connection_type)
+
+    def setblocking(self, blocking: bool) -> None:
         self.socket.setblocking(blocking)
 
-    def close(self):
+    def close(self) -> None:
         self.socket.close()
 
-    async def aio_close(self):
+    async def aio_close(self) -> None:
         if self._writer is None:
             return
         self._writer.close()
@@ -93,21 +120,21 @@ class ServiceConnection(object):
         self._writer = None
         self._reader = None
 
-    def recv(self, length=4096):
+    def recv(self, length=4096) -> bytes:
         """ socket.recv() normal behavior. attempt to receive a single chunk """
         return self.socket.recv(length)
 
-    def sendall(self, data):
+    def sendall(self, data: bytes) -> None:
         try:
             self.socket.sendall(data)
         except ssl.SSLEOFError as e:
             raise ConnectionTerminatedError from e
 
-    def send_recv_plist(self, data, endianity='>', fmt=plistlib.FMT_XML):
+    def send_recv_plist(self, data: Mapping, endianity='>', fmt=plistlib.FMT_XML) -> Mapping:
         self.send_plist(data, endianity=endianity, fmt=fmt)
         return self.recv_plist(endianity=endianity)
 
-    def recvall(self, size):
+    def recvall(self, size: int) -> bytes:
         data = b''
         while len(data) < size:
             chunk = self.recv(size - len(data))
@@ -116,11 +143,11 @@ class ServiceConnection(object):
             data += chunk
         return data
 
-    def recv_prefixed(self, endianity='>'):
+    def recv_prefixed(self, endianity='>') -> bytes:
         """ receive a data block prefixed with a u32 length field """
         size = self.recvall(4)
         if not size or len(size) != 4:
-            return
+            return b''
         size = struct.unpack(endianity + 'L', size)[0]
         while True:
             try:
@@ -129,13 +156,13 @@ class ServiceConnection(object):
                 # Allow ssl to do stuff
                 time.sleep(0)
 
-    async def aio_recv_prefixed(self, endianity='>'):
+    async def aio_recv_prefixed(self, endianity='>') -> bytes:
         """ receive a data block prefixed with a u32 length field """
         size = await self._reader.readexactly(4)
         size = struct.unpack(endianity + 'L', size)[0]
         return await self._reader.readexactly(size)
 
-    def send_prefixed(self, data):
+    def send_prefixed(self, data: bytes) -> None:
         """ send a data block prefixed with a u32 length field """
         if isinstance(data, str):
             data = data.encode()
@@ -143,30 +170,30 @@ class ServiceConnection(object):
         msg = b''.join([hdr, data])
         return self.sendall(msg)
 
-    def recv_plist(self, endianity='>'):
+    def recv_plist(self, endianity='>') -> Mapping:
         return parse_plist(self.recv_prefixed(endianity=endianity))
 
-    async def aio_recv_plist(self, endianity='>'):
+    async def aio_recv_plist(self, endianity='>') -> bytes:
         return parse_plist(await self.aio_recv_prefixed(endianity))
 
-    def send_plist(self, d, endianity='>', fmt=plistlib.FMT_XML):
+    def send_plist(self, d, endianity='>', fmt=plistlib.FMT_XML) -> None:
         return self.sendall(build_plist(d, endianity, fmt))
 
-    async def aio_send_plist(self, d, endianity='>', fmt=plistlib.FMT_XML):
+    async def aio_send_plist(self, d, endianity='>', fmt=plistlib.FMT_XML) -> None:
         self._writer.write(build_plist(d, endianity, fmt))
         await self._writer.drain()
 
-    def ssl_start(self, certfile, keyfile=None):
+    def ssl_start(self, certfile, keyfile=None) -> None:
         self.socket = create_context(certfile, keyfile=keyfile).wrap_socket(self.socket)
 
-    async def aio_ssl_start(self, certfile, keyfile=None):
+    async def aio_ssl_start(self, certfile, keyfile=None) -> None:
         self._reader, self._writer = await asyncio.open_connection(
             sock=self.socket,
             ssl=create_context(certfile, keyfile=keyfile),
             server_hostname=''
         )
 
-    def shell(self):
+    def shell(self) -> None:
         IPython.embed(
             header=highlight(SHELL_USAGE, lexers.PythonLexer(), formatters.TerminalTrueColorFormatter(style='native')),
             user_ns={
